/*
 * MIT License
 *
 * Copyright (c) 2019 Jakub Travnik
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package cz.jtra.math.quine.designer;

import java.awt.*;
import java.awt.event.MouseEvent;

/**
 */
public class CoordinateMapping {
    public static int MOUSE_CLOSE_CLICK_PIXELS = 5;
    private final ViewZoom viewZoom;
    private int w;
    private int h;
    private int inx1;
    private int inx2;
    private int iny1;
    private int iny2;
    private final int inw;
    private final int inh;

    public CoordinateMapping(Dimension size, ViewZoom viewZoom) {
        w = (int) Math.round(size.getWidth());
        h = (int) Math.round(size.getHeight());
        this.viewZoom = viewZoom;
        // inner square
        inw = Math.min(w, h);
        inh = Math.min(w, h);
        inx1 = (w - inw) / 2;
        inx2 = inx1 + inw;
        iny1 = (h - inh) / 2;
        iny2 = iny1 + inh;
    }

    public MousePos mouseEventToMousePos(MouseEvent mouseEvent) {
        int x = mouseEvent.getX();
        int y = mouseEvent.getY();
        if (x < inx2 && x >= inx1 && y < iny2 && y >= iny1) {
            double zeroToOneX = (x - inx1) / (double) inw;
            double zeroToOneY = 1 - ((y - iny1) / (double) inh);
            double vzW = viewZoom.getX2() - viewZoom.getX1();
            double vzH = viewZoom.getY2() - viewZoom.getY1();
            return new MousePos(viewZoom.getX1() + zeroToOneX * vzW,
                    viewZoom.getY1() + zeroToOneY * vzH,
                    MOUSE_CLOSE_CLICK_PIXELS / (double) inw * Math.sqrt(Math.pow(vzW, 2) + Math.pow(vzH, 2)),
                    mouseEvent.isShiftDown(), mouseEvent.isControlDown());
        } else {
            return null;
        }
    }

    public int getW() {
        return w;
    }

    public int getH() {
        return h;
    }

    public int getInx1() {
        return inx1;
    }

    public int getInx2() {
        return inx2;
    }

    public int getIny1() {
        return iny1;
    }

    public int getIny2() {
        return iny2;
    }
}
