/*
 * MIT License
 *
 * Copyright (c) 2019 Jakub Travnik
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package cz.jtra.math.quine.designer;

import javax.xml.bind.annotation.XmlType;
import java.util.Objects;

/**
 */
@XmlType(name = "Point")
public class Point implements Cloneable {
    public double x, y;
    public boolean selected;

    public Point() {
    }

    public Point(double x, double y) {
        this.x = x;
        this.y = y;
    }

    public void transform(TransformXY toX, TransformXY toY) {
        double xtmp = x;
        double ytmp = y;
        x = toX.transformXY(xtmp, ytmp);
        y = toY.transformXY(xtmp, ytmp);
    }

    public void moveTo(MousePos mousePos) {
        x = mousePos.x;
        y = mousePos.y;
    }

    public void moveRel(double ax, double ay) {
        x += ax;
        y += ay;
    }

    public void rotateCCW(MousePos pivot) {
        transform((x, y) -> -(y - pivot.y) + pivot.x, (x, y) -> (x - pivot.x) + pivot.y);
    }

    public void rotateCW(MousePos pivot) {
        transform((x, y) -> (y - pivot.y) + pivot.x, (x, y) -> -(x - pivot.x) + pivot.y);
    }

    public void flipLeftRight(MousePos pivot) {
        transform((x, y) -> -x + 2 * pivot.x, (x, y) -> y);
    }

    public void flipTopBottom(MousePos pivot) {
        transform((x, y) -> x, (x, y) -> -y + 2 * pivot.y);
    }


    @Override
    public String toString() {
        return "Point{" +
                "x=" + x +
                ", y=" + y +
                ", selected=" + selected +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Point point = (Point) o;
        return Double.compare(point.x, x) == 0 &&
                Double.compare(point.y, y) == 0;
    }

    @Override
    public int hashCode() {
        return Objects.hash(x, y);
    }

    public Point duplicate() {
        return new Point(x, y);
    }

}
