/*
 * MIT License
 *
 * Copyright (c) 2019 Jakub Travnik
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package cz.jtra.math.quine.designer;

/**
 */
public class ViewZoom {
    private double x1 = 0.0;
    private double x2 = 1.0;
    private double y1 = 0.0;
    private double y2 = 1.0;

    private EventQueue updateEventQueue = new EventQueue();

    public EventQueue getUpdateEventQueue() {
        return updateEventQueue;
    }


    public double getX1() {
        return x1;
    }

    public void setX1(double x1) {
        if (x2 > x1) {
            if (this.x1 != x1) {
                this.x1 = x1;
                updateEventQueue.postEvent();
            }
        }
    }

    public double getX2() {
        return x2;
    }

    public void setX2(double x2) {
        if (x2 > x1) {
            if (this.x2 != x2) {
                this.x2 = x2;
                updateEventQueue.postEvent();
            }
        }
    }

    public double getY1() {
        return y1;
    }

    public void setY1(double y1) {
        if (y2 > y1) {
            if (this.y1 != y1) {
                this.y1 = y1;
                updateEventQueue.postEvent();
            }
        }
    }

    public double getY2() {
        return y2;
    }

    public void setY2(double y2) {
        if (y2 > y1) {
            if (this.y2 != y2) {
                this.y2 = y2;
                updateEventQueue.postEvent();
            }
        }
    }

    public void setAll(double x1, double x2, double y1, double y2) {
        if (y2 > y1 && x2 > x1) {
            boolean update = false;
            if (this.x1 != x1) {
                this.x1 = x1;
                update = true;
            }
            if (this.x2 != x2) {
                this.x2 = x2;
                update = true;
            }
            if (this.y1 != y1) {
                this.y1 = y1;
                update = true;
            }
            if (this.y2 != y2) {
                this.y2 = y2;
                update = true;
            }
            if (update) {
                updateEventQueue.postEvent();
            }
        }
    }


    public void magnify(double factor) {
        double centerx = (x1 + x2) / 2;
        double centery = (y1 + y2) / 2;
        double w = x2 - x1;
        double h = y2 - y1;
        x1 = centerx - w / 2 / factor;
        y1 = centery - h / 2 / factor;
        x2 = centerx + w / 2 / factor;
        y2 = centery + h / 2 / factor;
        updateEventQueue.postEvent();
    }
}
